@@ -93,6 +93,7 @@ extern char VERSION_INFO[256];
 #define SC_NORMAL  0
 #define SC_FIRERUN 1
 #define SC_ALWAYS  2
+#define SC_MOVETO  3
 /*@}*/
 
 typedef struct Animations {
@@ -110,6 +111,9 @@ typedef struct Animations {
 
 extern Animations animations[MAXANIM];
 
+/** Largest ncom command number to send before wrapping. */
+#define COMMAND_MAX 255
+
 /**
  * Basic support for socket communications, including the file descriptor,
  * input buffer, server, server, version, etc. ClientSocket could probably
@@ -129,6 +133,7 @@ typedef struct {
                                          *   currently take to execute
                                          */
     char* servername;
+    gint8 dir[COMMAND_MAX];    //< Direction of sent movement command (or -1) for local prediction
 } ClientSocket;
 
 extern ClientSocket csocket;
