@@ -25,11 +25,20 @@
 #include "mapdata.h"
 
 /**
- * Position of a player on the map, relative to the coordinates received
- * after a 'newmap' command. These are kept to for reporting to client scripts.
+ * Position of the player on the internal map. Internal to the client.
+ */
+PlayerPosition pl_pos;
+
+/**
+ * Position of the player reported to client scripts. It resets to (0, 0)
+ * after a 'newmap' command.
  */
 PlayerPosition script_pos;
 
+/** Move to coordinates on the current map. (0, 0) means no destination. */
+int move_to_x = 0, move_to_y = 0;
+bool move_to_attack = false;
+
 /**
  * Size of virtual map.
  */
@@ -77,15 +86,18 @@ struct BigCell {
     guint8 layer;
 };
 
+/* Global map rendering offsets used for local scroll prediction. */
+int global_offset_x = 0;
+int global_offset_y = 0;
+int want_offset_x = 0;
+int want_offset_y = 0;
 
 static void recenter_virtual_map_view(int diff_x, int diff_y);
 static void mapdata_get_image_size(int face, guint8 *w, guint8 *h);
 
-
 static int width;   //< width of current map view
 static int height;  //< height of current map view
 
-
 /**
  * Contains the head of a list of all currently active big faces outside the
  * view area. All entries are part of bigfaces[].
@@ -170,27 +182,6 @@ static void set_darkness(int x, int y, int darkness)
 
     mapdata_cell(x, y)->darkness = darkness;
     mapdata_cell(x, y)->need_update = 1;
-
-    /* pretty ugly - since the light code with pngximage uses neighboring
-     * spaces to adjust the darkness, we now need to let the neighbors know
-     * they should update their darkness now.
-     */
-    if (use_config[CONFIG_DISPLAYMODE] == CFG_DM_SDL
-            && (use_config[CONFIG_LIGHTING] == CFG_LT_PIXEL
-                ||  use_config[CONFIG_LIGHTING] == CFG_LT_PIXEL_BEST)) {
-        if (x > 1) {
-            mapdata_cell(x-1, y)->need_update = 1;
-        }
-        if (y > 1) {
-            mapdata_cell(x, y-1)->need_update = 1;
-        }
-        if (x < width-1) {
-            mapdata_cell(x+1, y)->need_update = 1;
-        }
-        if (y < height-1) {
-            mapdata_cell(x, y+1)->need_update = 1;
-        }
-    }
 }
 
 static void mark_resmooth(int x, int y, int layer)
@@ -614,12 +605,19 @@ static void mapdata_init(void)
         }
     }
     bigfaces_head = NULL;
+
+    global_offset_x = 0;
+    global_offset_y = 0;
+    want_offset_x = 0;
+    want_offset_y = 0;
 }
 
 void mapdata_free(void) {
     if (the_map._cells != NULL) {
         g_free(the_map._cells);
         the_map._cells = NULL;
+        the_map.width = 0;
+        the_map.height = 0;
     }
 }
 
@@ -991,6 +989,8 @@ void mapdata_scroll(int dx, int dy)
     while (bigfaces_head != NULL) {
         expand_clear_bigface_from_layer(bigfaces_head->x, bigfaces_head->y, bigfaces_head->layer, 0);
     }
+
+    run_move_to();
 }
 
 void mapdata_newmap(void)
@@ -999,6 +999,14 @@ void mapdata_newmap(void)
     script_pos.y = 0;
     int x, y;
 
+    global_offset_x = 0;
+    global_offset_y = 0;
+    want_offset_x = 0;
+    want_offset_y = 0;
+
+    // Clear past predictions.
+    memset(csocket.dir, -1, sizeof(csocket.dir));
+
     /* Clear the_map.cells[]. */
     for (x = 0; x < the_map.width; x++) {
         clear_cells(x, 0, the_map.height);
@@ -1011,6 +1019,9 @@ void mapdata_newmap(void)
     while (bigfaces_head != NULL) {
         expand_clear_bigface_from_layer(bigfaces_head->x, bigfaces_head->y, bigfaces_head->layer, 0);
     }
+
+    // Clear destination.
+    clear_move_to();
 }
 
 /**
@@ -1435,3 +1446,72 @@ void mapdata_animation(void)
         }
     }
 }
+
+/**
+ * Compute player position in map coordinates. Unlike pl_pos.x and pl_pos.y
+ * which store the top left corner of the virtual map, this returns the actual
+ * position of the player.
+ */
+void pl_mpos(int *px, int *py) {
+    const int vw = use_config[CONFIG_MAPWIDTH];
+    const int vh = use_config[CONFIG_MAPHEIGHT];
+    *px = pl_pos.x + vw/2;
+    *py = pl_pos.y + vh/2;
+}
+
+void set_move_to(int dx, int dy) {
+    int old_move_to_x = move_to_x;
+    int old_move_to_y = move_to_y;
+
+    pl_mpos(&move_to_x, &move_to_y);
+    move_to_x += dx;
+    move_to_y += dy;
+
+    if (move_to_x == old_move_to_x && move_to_y == old_move_to_y) {
+        // Detect double-click.
+        move_to_attack = true;
+    } else {
+        move_to_attack = false;
+    }
+}
+
+void clear_move_to() {
+    move_to_x = 0;
+    move_to_y = 0;
+    move_to_attack = false;
+}
+
+bool is_at_moveto() {
+    if (move_to_x == 0 && move_to_y == 0) {
+        return true;
+    }
+
+    int px, py;
+    pl_mpos(&px, &py);
+    return !(move_to_x != px || move_to_y != py);
+}
+
+void run_move_to() {
+    if (move_to_x == 0 && move_to_y == 0) {
+        // If not moving to a tile, skip to avoid calling stop_run().
+        return;
+    }
+
+    if (is_at_moveto()) {
+        clear_move_to();
+        stop_run();
+        return;
+    }
+
+    int px, py;
+    pl_mpos(&px, &py);
+    int dx = move_to_x - px;
+    int dy = move_to_y - py;
+    int dir = relative_direction(dx, dy);
+
+    if (move_to_attack) {
+        run_dir(dir);
+    } else {
+        walk_dir(dir);
+    }
+}
