@@ -31,6 +31,7 @@
 #include "client.h"
 #include "external.h"
 #include "script.h"
+#include "mapdata.h"
 
 bool profile_latency = false;
 int64_t *profile_time = NULL; /** 256-length array to keep track of when
@@ -148,6 +149,66 @@ void run_dir(int dir) {
     }
 }
 
+extern const char *const directions[];
+
+int command_to_direction(const char *dir) {
+    for (int i = 0; i < 9; i++) {
+        if (!strcmp(dir, directions[i])) {
+            return i;
+        }
+    }
+    return -1;
+}
+
+const char* dir_to_command(int dir) {
+    return directions[dir];
+}
+
+void walk_dir(int dir) {
+    send_command(dir_to_command(dir), -1, SC_MOVETO);
+}
+
+/**
+ * Change map drawing offset to provide local movement prediction based on
+ * player's movement commands to the server.
+ */
+void predict_scroll(int dir) {
+    switch (dir%8) {
+    case 0:
+        want_offset_x += 1;
+        want_offset_y += 1;
+        break;
+    case 1:
+        want_offset_x += 0;
+        want_offset_y += 1;
+        break;
+    case 2:
+        want_offset_x += -1;
+        want_offset_y += 1;
+        break;
+    case 3:
+        want_offset_x += -1;
+        want_offset_y += 0;
+        break;
+    case 4:
+        want_offset_x += -1;
+        want_offset_y += -1;
+        break;
+    case 5:
+        want_offset_x += 0;
+        want_offset_y += -1;
+        break;
+    case 6:
+        want_offset_x += 1;
+        want_offset_y += -1;
+        break;
+    case 7:
+        want_offset_x += 1;
+        want_offset_y += 0;
+        break;
+    }
+}
+
 /* This should be used for all 'command' processing.  Other functions should
  * call this so that proper windowing will be done.
  * command is the text command, repeat is a count value, or -1 if none
@@ -168,6 +229,10 @@ int send_command(const char *command, int repeat, int must_send) {
         return 0;
     }
 
+    if (use_config[CONFIG_ECHO]) {
+        draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE, command);
+    }
+
     /* Does the server understand 'ncom'? If so, special code */
     if (csocket.cs_version >= 1021) {
         int commdiff=csocket.command_sent - csocket.command_received;
@@ -197,7 +262,7 @@ int send_command(const char *command, int repeat, int must_send) {
                 strcpy(last_command, command);
             }
             csocket.command_sent++;
-            csocket.command_sent &= 0xff;   /* max out at 255 */
+            csocket.command_sent %= COMMAND_MAX;
 
             SockList_Init(&sl, buf);
             SockList_AddString(&sl, "ncom ");
@@ -212,6 +277,16 @@ int send_command(const char *command, int repeat, int must_send) {
                 profile_time[csocket.command_sent] = g_get_monotonic_time();
                 printf("profile/com\t%d\t%s\n", csocket.command_sent, command);
             }
+
+            int dir = command_to_direction(command);
+            csocket.dir[csocket.command_sent] = dir;
+            if (drun == -1 && dir != -1) {
+                // If movement command, predict scroll.
+                predict_scroll(dir);
+                if (must_send != SC_MOVETO) {
+                    clear_move_to();
+                }
+            }
         }
     } else {
         cs_print_string(csocket.fd, "command %d %s", repeat,command);
@@ -239,6 +314,13 @@ void CompleteCmd(unsigned char *data, int len) {
                    csocket.command_time, in_flight);
         }
     }
+    int dir = csocket.dir[csocket.command_received];
+    if (drun == -1 && dir != -1) {
+        // Revert prediction when command is acknowledged. Note that we undo
+        // the prediction the same regardless of whether the command succeeded
+        // or not, because the player always ends up in the center of the map.
+        predict_scroll(dir+4);
+    }
     script_sync(in_flight);
 }
 
