@@ -39,10 +39,26 @@ GtkComboBoxText *config_combobox_faceset;
 GtkComboBox *config_combobox_displaymode, *config_combobox_lighting;
 
 #define THEME_DEFAULT CF_DATADIR "/themes/Standard"
-static char *theme = THEME_DEFAULT;
+
+/* Configuration variables initialized to NULL, set by config_load() */
+static char *theme;
+char* last_server;
+
+/*
+ * This should really be one of the CONFIG values, or perhaps a checkbox
+ * someplace that displays frame rate.
+ */
+bool time_map_redraw = false;
+
+/** Speed of local map prediction scrolling, 0-100 (0 to disable). */
+int predict_alpha;
 
 static void on_config_close(GtkButton *button, gpointer user_data);
 
+static bool IS_DIFFERENT(int type) {
+    return want_config[type] != use_config[type];
+}
+
 /**
  * Return the basename of the current UI file.
  */
@@ -152,7 +168,8 @@ void load_theme(int reload) {
      * <layout>.gtkrc files.  Remember, strcmp returns zero on a match, and
      * a theme file should not be registered if "None" is selected.
      */
-    if (theme != NULL) {
+    g_assert(theme != NULL); // ensured by config_load()
+    {
         /*
          * Check for existence of the client theme file.  Unfortunately, at
          * initial run time, the window may not be realized yet, so the
@@ -162,7 +179,8 @@ void load_theme(int reload) {
          */
         if (access(theme, R_OK) == -1) {
             LOG(LOG_ERROR, "load_theme", "Unable to find theme file %s", theme);
-            theme = THEME_DEFAULT;
+            g_free(theme);
+            theme = g_strdup(THEME_DEFAULT);
         }
         gtk_rc_add_default_file(theme);
     }
@@ -260,7 +278,10 @@ static void config_load_legacy() {
                     "Malformed mapsize option in gdefaults2.  Ignoring");
             }
         } else if (!strcmp(inbuf, "theme")) {
-            theme = g_strdup(cp);   /* memory leak ! */
+            if (theme != NULL) {
+                g_free(theme);
+            }
+            theme = g_strdup(cp);
             continue;
         } else if (!strcmp(inbuf, "window_layout")) {
             strncpy(window_xml_file, cp, MAX_BUF - 1);
@@ -303,10 +324,9 @@ static void config_load_legacy() {
 }
 
 /**
- * Sanity check values set in want_config and copy them over to use_config
- * when all of them are acceptable.
- *
- * This function should be called after config_load() and parse_args().
+ * Check that want_config is valid, copy the new configuration to use_config,
+ * and apply the new configuration. Call this after changing want_config,
+ * either through config_load() or on_config_close().
  */
 void config_check() {
     if (want_config[CONFIG_ICONSCALE] < 25 ||
@@ -318,11 +338,10 @@ void config_check() {
         want_config[CONFIG_ICONSCALE] = use_config[CONFIG_ICONSCALE];
     }
 
-    if (want_config[CONFIG_MAPSCALE] < 25 ||
-            want_config[CONFIG_MAPSCALE] > 200) {
+    if (want_config[CONFIG_MAPSCALE] < 10 || want_config[CONFIG_MAPSCALE] > 200) {
         LOG(LOG_WARNING, "config_check",
                 "Ignoring invalid 'mapscale' value '%d'; "
-                "must be between 25 and 200.\n",
+                "must be between 10 and 200.\n",
                 want_config[CONFIG_MAPSCALE]);
         want_config[CONFIG_MAPSCALE] = use_config[CONFIG_MAPSCALE];
     }
@@ -376,12 +395,42 @@ void config_check() {
     }
 #endif
 
+    if (want_config[CONFIG_CACHE]) {
+        LOG(LOG_ERROR, "config_check",
+                "Image caching is not currently supported in this client. Running without image caching.");
+        want_config[CONFIG_CACHE] = 0;
+    }
+
+    // Enable darkness if lighting is not 'None'.
+    if (want_config[CONFIG_LIGHTING] != CFG_LT_NONE) {
+        want_config[CONFIG_DARKNESS] = 1;
+    }
+
+    if (want_config[CONFIG_SOUND] && init_sounds()) {
+        use_config[CONFIG_SOUND] = 1;
+    } else {
+        use_config[CONFIG_SOUND] = 0;
+    }
+    if (csocket.fd) {
+        cs_print_string(csocket.fd, "setup sound %d", use_config[CONFIG_SOUND]);
+    }
+
+#ifdef TCP_NODELAY
+#ifndef WIN32
+    // TODO: Merge with setsockopt code from client.c
+    int q = want_config[CONFIG_FASTTCP];
+
+    if (csocket.fd && setsockopt(csocket.fd, SOL_TCP, TCP_NODELAY, &q, sizeof(q)) == -1) {
+        perror("TCP_NODELAY");
+    }
+#endif
+#endif
+
     /* Copy sanitized user settings to current settings. */
     memcpy(use_config, want_config, sizeof(use_config));
 
+    map_check_resize();
     image_size = DEFAULT_IMAGE_SIZE * use_config[CONFIG_ICONSCALE] / 100;
-    map_image_size = DEFAULT_IMAGE_SIZE * use_config[CONFIG_MAPSCALE] / 100;
-    map_image_half_size = DEFAULT_IMAGE_SIZE * use_config[CONFIG_MAPSCALE] / 200;
     if (!use_config[CONFIG_CACHE]) {
         use_config[CONFIG_DOWNLOAD] = FALSE;
     }
@@ -413,14 +462,24 @@ void config_load() {
         }
 
         /* Load additional settings. */
-        /* TODO: Both of these below are one-time memory leaks. */
-        theme = g_key_file_get_string(config, "GTKv2",
-                "theme", NULL);
-        face_info.want_faceset = g_key_file_get_string(config, "GTKv2",
-                "faceset", NULL);
+        if (theme != NULL) {
+            g_free(theme);
+        }
+        theme = g_key_file_get_string(config, "GTKv2", "theme", NULL);
 
-        char *layout =
-            g_key_file_get_string(config, "GTKv2", "window_layout", NULL);
+        if (face_info.want_faceset != NULL) {
+            g_free(face_info.want_faceset);
+        }
+        face_info.want_faceset = g_key_file_get_string(config, "GTKv2", "faceset", NULL);
+
+        predict_alpha = g_key_file_get_integer(config, "GTKv2", "predict_alpha", NULL);
+
+        if (last_server != NULL) {
+            g_free(last_server);
+        }
+        last_server = g_key_file_get_string(config, "GTKv2", "last_server", NULL);
+
+        char *layout = g_key_file_get_string(config, "GTKv2", "window_layout", NULL);
         g_strlcpy(window_xml_file, layout, sizeof(window_xml_file));
         free(layout);
     } else {
@@ -429,6 +488,18 @@ void config_load() {
         /* Load legacy configuration file. */
         config_load_legacy();
     }
+
+    if (theme == NULL) {
+        theme = g_strdup(THEME_DEFAULT);
+    }
+
+    if (face_info.want_faceset == NULL) {
+        face_info.want_faceset = g_strdup("");
+    }
+
+    if (last_server == NULL) {
+        last_server = g_strdup("");
+    }
 }
 
 /**
@@ -441,6 +512,8 @@ void save_defaults() {
     /* Save GTKv2 specific client settings. */
     g_key_file_set_string(config, "GTKv2", "theme", theme);
     g_key_file_set_string(config, "GTKv2", "faceset", face_info.want_faceset);
+    g_key_file_set_string(config, "GTKv2", "last_server", last_server);
+    g_key_file_set_integer(config, "GTKv2", "predict_alpha", predict_alpha);
     g_key_file_set_string(config, "GTKv2", "window_layout", window_xml_file);
 
     /* Save the rest of the client settings. */
@@ -605,8 +678,6 @@ static void setup_config_dialog() {
     gtk_file_chooser_set_filename(theme_filechooser, theme);
 }
 
-#define IS_DIFFERENT(TYPE) (want_config[TYPE] != use_config[TYPE])
-
 /**
  * Get an integer value from 'column' of the active field in 'combobox'.
  */
@@ -680,12 +751,6 @@ static void read_config_dialog(void) {
     want_config[CONFIG_LIGHTING] =
         gtk_combo_box_get_active(config_combobox_lighting);
 
-    // Enable darkness if lighting is not 'None'.
-    if (want_config[CONFIG_LIGHTING] != CFG_LT_NONE) {
-        want_config[CONFIG_DARKNESS] = 1;
-        use_config[CONFIG_DARKNESS] = 1;
-    }
-
     // Set UI file.
     buf = gtk_file_chooser_get_filename(ui_filechooser);
     if (buf != NULL) {
@@ -701,54 +766,7 @@ static void read_config_dialog(void) {
         load_theme(TRUE);
     }
 
-    /*
-     * Some values can take effect right now, others not.  Code below handles
-     * these cases - largely grabbed from gtk/config.c
-     */
-    if (IS_DIFFERENT(CONFIG_SOUND)) {
-        int tmp;
-        if (want_config[CONFIG_SOUND]) {
-            tmp = init_sounds();
-            if (csocket.fd) {
-                cs_print_string(csocket.fd, "setup sound %d", tmp >= 0);
-            }
-        } else {
-            if (csocket.fd) {
-                cs_print_string(csocket.fd, "setup sound 0");
-            }
-        }
-        use_config[CONFIG_SOUND] = want_config[CONFIG_SOUND];
-    }
-    if (IS_DIFFERENT(CONFIG_FASTTCP)) {
-#ifdef TCP_NODELAY
-#ifndef WIN32
-        // TODO: Merge with setsockopt code from client.c
-        int q = want_config[CONFIG_FASTTCP];
-
-        if (csocket.fd &&
-                setsockopt(csocket.fd, SOL_TCP, TCP_NODELAY, &q, sizeof(q)) == -1) {
-            perror("TCP_NODELAY");
-        }
-#endif
-#endif
-        use_config[CONFIG_FASTTCP] = want_config[CONFIG_FASTTCP];
-    }
-
-    if (IS_DIFFERENT(CONFIG_LIGHTING)) {
-#ifdef HAVE_SDL
-        if (use_config[CONFIG_DISPLAYMODE] == CFG_DM_SDL)
-            /* This is done to make the 'lightmap' in the proper format */
-        {
-            init_SDL(NULL, 1);
-        }
-#endif
-    }
-    /*
-     * Nothing to do, but we can switch immediately without problems.  do force
-     * a redraw
-     */
     if (IS_DIFFERENT(CONFIG_GRAD_COLOR)) {
-        use_config[CONFIG_GRAD_COLOR] = want_config[CONFIG_GRAD_COLOR];
         draw_stats(TRUE);
     }
 }
@@ -760,6 +778,7 @@ void on_configure_activate(GtkWidget *menuitem, gpointer user_data) {
 
 static void on_config_close(GtkButton *button, gpointer user_data) {
     read_config_dialog();
+    config_check();
     save_defaults();
     gtk_widget_hide(config_dialog);
 }
