@@ -204,66 +204,81 @@ static int get_item_env(item *it) {
     return (ITEM_IN_CONTAINER | get_item_env(it->env));
 }
 
+static void list_item_drop(item *tmp);
+
 /**
  *
  * @param event
  * @param tmp
  */
 static void list_item_action(GdkEventButton *event, item *tmp) {
-    int env;
-
-    /* We need to know where this item is in fact is */
-    env = get_item_env(tmp);
-
     /* It'd sure be nice if these weren't hardcoded values for button and
      * shift presses.
      */
-    if (event->button == 1) {
+    if (event->button == 1) { // primary/left mouse button
         if (event->state & GDK_SHIFT_MASK) {
             toggle_locked(tmp);
         } else {
             client_send_examine(tmp->tag);
         }
-    } else if (event->button == 2) {
+    } else if (event->button == 2) { // middle mouse button
+        /* Some mice do not have a functioning middle mouse button,
+         * so all the actions here are duplicated elsewhere
+         */
         if (event->state & GDK_SHIFT_MASK) {
             send_mark_obj(tmp);
         } else {
             client_send_apply(tmp->tag);
         }
-    } else if (event->button == 3) {
-        if (tmp->locked) {
-            draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
-                    "This item is locked. To drop it, first unlock by shift+leftclicking on it.");
+    } else if (event->button == 3) { // secondary/right mouse button
+        if (event->state & GDK_SHIFT_MASK) {
+            client_send_apply(tmp->tag);
+        } else if (event->state & GDK_CONTROL_MASK) {
+            send_mark_obj(tmp);
         } else {
-            guint32 dest;
-
-            cpl.count = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_count));
-            /*
-             * Figure out where to move the item to.  If it is on the ground,
-             * it is moving to the players inventory.  If it is in a container,
-             * it is also moving to players inventory.  If it is in the players
-             * inventory (not a container) and the player has an open container
-             * in his inventory, move the object to the container (not ground).
-             * Otherwise, it is moving to the ground (dest=0).  Have to look at
-             * the item environment, because what list is no longer accurate.
-             */
-            if (env & (ITEM_GROUND | ITEM_IN_CONTAINER)) {
-                dest = cpl.ob->tag;
-            } else if (env == ITEM_INVENTORY && cpl.container &&
-                    (get_item_env(cpl.container) == ITEM_INVENTORY ||
-                    get_item_env(cpl.container) == ITEM_GROUND)) {
-                dest = cpl.container->tag;
-            } else {
-                dest = 0;
-            }
-
-            client_send_move(dest, tmp->tag, cpl.count);
-            gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton_count), 0.0);
-            cpl.count = 0;
+            list_item_drop(tmp);
         }
     }
 }
 
+static void list_item_drop(item *tmp) {
+    int env;
+
+    /* We need to know where this item is in fact is */
+    env = get_item_env(tmp);
+
+    if (tmp->locked) {
+        draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
+                "This item is locked. To drop it, first unlock by shift+leftclicking on it.");
+    } else {
+        guint32 dest;
+
+        cpl.count = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_count));
+        /*
+         * Figure out where to move the item to.  If it is on the ground,
+         * it is moving to the players inventory.  If it is in a container,
+         * it is also moving to players inventory.  If it is in the players
+         * inventory (not a container) and the player has an open container
+         * in his inventory, move the object to the container (not ground).
+         * Otherwise, it is moving to the ground (dest=0).  Have to look at
+         * the item environment, because what list is no longer accurate.
+         */
+        if (env & (ITEM_GROUND | ITEM_IN_CONTAINER)) {
+            dest = cpl.ob->tag;
+        } else if (env == ITEM_INVENTORY && cpl.container &&
+                (get_item_env(cpl.container) == ITEM_INVENTORY ||
+                get_item_env(cpl.container) == ITEM_GROUND)) {
+            dest = cpl.container->tag;
+        } else {
+            dest = 0;
+        }
+
+        client_send_move(dest, tmp->tag, cpl.count);
+        gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton_count), 0.0);
+        cpl.count = 0;
+    }
+}
+
 /**
  * Used when a button is pressed on the inventory or look list.  The parameters
  * are those determined by the callback.  Note that this function isn't 100%
@@ -667,7 +682,31 @@ static GtkStyle *get_row_style(item *it) {
  *    insertions and removals are more difficult.
  */
 
-void item_event_item_deleting(item * it) {
+static void remove_object_from_store(item* it, GtkTreeStore* store) {
+    GtkTreeIter iter;
+    gboolean valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+    while (valid) {
+        item* curr_item;
+        gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, LIST_OBJECT, &curr_item, -1);
+        if (curr_item == it) {
+            gtk_tree_store_remove(store, &iter);
+            break;
+        }
+        valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
+    }
+}
+
+void item_event_item_deleting(item* it) {
+    switch (get_item_env(it)) {
+    case ITEM_INVENTORY:
+        remove_object_from_store(it, treestore);
+        cpl.ob->inv_updated = 1;
+        break;
+    case ITEM_GROUND:
+        remove_object_from_store(it, store_look);
+        cpl.below->inv_updated = 1;
+        break;
+    }
 }
 
 void item_event_container_clearing(item * container) {
@@ -852,6 +891,13 @@ static gboolean drawingarea_inventory_table_expose_event(GtkWidget *widget,
         GdkEventExpose *event, gpointer user_data) {
     item *tmp;
 
+    if (cpl.ob->inv_updated != 0) {
+        // Delay drawing until inventory is fully updated. This avoids drawing
+        // previously added items that may now be removed, leading to a heap
+        // use-after-free.
+        return TRUE;
+    }
+
     tmp = (item*) user_data;
 
     /*
