@@ -412,7 +412,7 @@ static void parse_keybind_line(char *buf, int line, unsigned int scope_flag) {
         keysym = gdk_keyval_from_name(buf);
         if (!keysym) {
             LOG(LOG_WARNING, "gtk-v2::parse_keybind_line",
-                "Unable to convert line %d (%s) into keysym", line, cp);
+                "Unable to convert line %d (%s) into keysym", line, buf);
             return;
         }
         cp = cpnext;
@@ -482,6 +482,10 @@ static void parse_keybind_line(char *buf, int line, unsigned int scope_flag) {
         }
 
         flags |= scope_flag;  /* add the corresponding scope flag */
+
+        // If there is a trailing CR, remove it.
+        cpnext[strcspn(cpnext, "\r")] = 0;
+
         keybind_insert(keysym, flags, cpnext);
 
     } /* else if not special binding line */
@@ -595,22 +599,38 @@ void keybindings_init(const char *character_name) {
     /* Try to load global keybindings. */
     snprintf(buf, sizeof(buf), "%s/keys", config_dir);
     GFile *f = g_file_new_for_path(buf);
-    GInputStream *in = G_INPUT_STREAM(g_file_read(f, NULL, NULL));
-    if (in != NULL) {
-        LOG(LOG_DEBUG, "keybindings_init", "Loading global keybindings");
-        parse_keys_file(in, KEYF_R_GLOBAL);
+    GFileInputStream * fs = g_file_read(f, NULL, NULL);
+    if (fs != NULL) {
+        GInputStream *in = G_INPUT_STREAM(fs);
+        if (in != NULL) {
+            LOG(LOG_DEBUG, "keybindings_init", "Loading global keybindings");
+            parse_keys_file(in, KEYF_R_GLOBAL);
+            g_object_unref(in);
+        }
+        g_object_unref(fs);
     }
+    g_object_unref(f);
 
     /* Try to load the character-specific keybindings. */
     if (cpl.name) {
         snprintf(buf, sizeof(buf), "%s/%s.%s.keys", config_dir,
                 csocket.servername, cpl.name);
         GFile *f = g_file_new_for_path(buf);
-        GInputStream *in = G_INPUT_STREAM(g_file_read(f, NULL, NULL));
-        if (in != NULL) {
-            LOG(LOG_DEBUG, "keybindings_init", "Loading character keybindings");
-            parse_keys_file(in, KEYF_R_CHAR);
+        GFileInputStream * fs = g_file_read(f, NULL, NULL);
+        if (fs != NULL) {
+            GInputStream *in = G_INPUT_STREAM(fs);
+            if (in != NULL) {
+                LOG(LOG_DEBUG, "keybindings_init", "Loading character keybindings for '%s'", cpl.name);
+                parse_keys_file(in, KEYF_R_CHAR);
+                g_object_unref(in);
+            }
+            g_object_unref(fs);
+        } else {
+            LOG(LOG_DEBUG, "keybindings_init", "No character keybindings for '%s' found", cpl.name);
         }
+        g_object_unref(f);
+    } else {
+        LOG(LOG_DEBUG, "keybindings_init", "No character name");
     }
 }
 
@@ -789,9 +809,6 @@ static void parse_key_release(guint32 keysym) {
         gtk_label_set_text(GTK_LABEL(fire_label), "    ");
     } else if (keysym == runkeysym[0] || keysym == runkeysym[1]) {
         cpl.run_on = 0;
-        if (use_config[CONFIG_ECHO])
-            draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
-                          "stop run");
         clear_run();
         gtk_label_set_text(GTK_LABEL(run_label), "   ");
     } else if (keysym == altkeysym[0] || keysym == altkeysym[1]) {
@@ -892,15 +909,7 @@ static void parse_key(char key, guint32 keysym) {
             } else {
                 extended_command(kb->command);
             }
-            if (use_config[CONFIG_ECHO])
-                draw_ext_info(
-                    NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
-                    kb->command);
         } else {
-            if (use_config[CONFIG_ECHO])
-                draw_ext_info(
-                    NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
-                    kb->command);
             extended_command(kb->command);
         }
         return;
@@ -1522,9 +1531,6 @@ void focusoutfunc(GtkWidget *widget, GdkEventKey *event, GtkWidget *window) {
     }
     if (cpl.run_on == 1) {
         cpl.run_on = 0;
-        if (use_config[CONFIG_ECHO])
-            draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
-                          "stop run");
         clear_run();
         gtk_label_set_text(GTK_LABEL(run_label), "   ");
     }
