@@ -80,6 +80,8 @@ static GOptionEntry options[] = {
     { "updatekeycodes", 0, 0, G_OPTION_ARG_NONE, &updatekeycodes,
         "Update the saved bindings for this keyboard", NULL },
 
+    { "loginmethod", 0, 0, G_OPTION_ARG_INT, &wantloginmethod,
+        "Login method to request from server", NULL },
     { "profile-latency", 0, 0, G_OPTION_ARG_NONE, &profile_latency,
         "Log command acknowledgement latency to stdout", NULL },
     { "profile-redraw", 0, 0, G_OPTION_ARG_NONE, &time_map_redraw,
@@ -130,12 +132,9 @@ static gboolean redraw(gpointer data) {
                 cpl.container->inv_updated = 1;
             }
             cpl.ob->inv_updated = 1;
-
             have_new_image = 0;
-            draw_map(1);
-        } else {
-            draw_map(0);
         }
+        draw_map();
         draw_lists();
     }
     return FALSE;
@@ -197,7 +196,10 @@ static gboolean do_network(GObject *stream, gpointer data) {
     script_fdset(&maxfd, &tmp_read);
     pollret = select(maxfd, &tmp_read, NULL, NULL, &timeout);
     if (pollret < 0) {
+#ifndef WIN32
+		// FIXME: For whatever reason, we get errors claiming "no error" here in Windows
         LOG(LOG_ERROR, "do_network", "script select() failed: %s", strerror(errno));
+#endif
     } else if (pollret > 0) {
         script_process(&tmp_read);
     }
@@ -210,7 +212,7 @@ static gboolean do_network(GObject *stream, gpointer data) {
  */
 static void event_loop() {
 #ifdef WIN32
-    g_timeout_add(250, (GtkFunction) do_scriptout, NULL);
+    g_timeout_add(250, G_SOURCE_FUNC(do_scriptout), NULL);
 #endif
 
     GSource *net_source = client_get_source();
@@ -297,9 +299,6 @@ void my_log_handler(const gchar *log_domain, GLogLevelFlags log_level,
 }
 
 static void init_sockets() {
-    /* Use the 'new' login method. */
-    wantloginmethod = 2;
-
 #ifndef WIN32
     signal(SIGPIPE, SIG_IGN);
 #endif
@@ -338,7 +337,7 @@ static void init_ui() {
     /* Load main window using GtkBuilder. */
     window_xml = gtk_builder_new();
     if (init_ui_layout(window_xml_file) == NULL) {
-        LOG(LOG_DEBUG, "init_ui_layout", "Using default layout");
+        LOG(LOG_DEBUG, "init_ui_layout", "Could not initialize '%s', using default layout", window_xml_file);
         if (init_ui_layout(DEFAULT_UI) == NULL) {
             g_error("Could not load default layout!");
         }
