@@ -13,12 +13,13 @@
 
 /**
  * @file
- * Handles map related code in terms of allocation, insertion of new objects,
- * and actual rendering (although the sdl rendering is in the sdl file
+ * Initializes and renders the map drawing area. Handles map drawing area
+ * events, notably the "click to look at" mouse event.
  */
 
 #include "client.h"
 
+#include <math.h>
 #include <gtk/gtk.h>
 
 #include "image.h"
@@ -26,33 +27,31 @@
 #include "mapdata.h"
 #include "gtk2proto.h"
 
+/* Configuration (from config.c) */
+extern int predict_alpha;
+extern bool time_map_redraw;
+
+int map_image_size = DEFAULT_IMAGE_SIZE;
 static gboolean map_updated = FALSE;
 
-// Declarations for local event-handling functions.
+GtkWidget *map_notebook;
+static GtkWidget *map_drawing_area;
+
+// Forward declarations for events
 static gboolean map_button_event(GtkWidget *widget,
         GdkEventButton *event, gpointer user_data);
 static gboolean map_expose_event(GtkWidget *widget,
         GdkEventExpose *event, gpointer user_data);
 
-PlayerPosition pl_pos;
-
-int map_image_size = DEFAULT_IMAGE_SIZE;
-int map_image_half_size = DEFAULT_IMAGE_SIZE / 2;
-
-static GtkWidget *map_drawing_area;
-
-GtkWidget *map_notebook;
-
-/*
- * This should really be one of the CONFIG values, or perhaps a checkbox
- * someplace that displays frame rate.
- */
-bool time_map_redraw = false;
-
 /**
  * Calculate and set desired map size based on map window size.
  */
-static void map_check_resize() {
+void map_check_resize() {
+    if (!GTK_IS_WIDGET(map_drawing_area)) {
+        // Called by config_check(), but main window layout not yet loaded.
+        return;
+    }
+
     GtkAllocation size;
     gtk_widget_get_allocation(map_drawing_area, &size);
     int scaled_size = map_image_size * use_config[CONFIG_MAPSCALE] / 100;
@@ -61,10 +60,21 @@ static void map_check_resize() {
     w = (w > MAP_MAX_SIZE) ? MAP_MAX_SIZE : w;
     h = (h > MAP_MAX_SIZE) ? MAP_MAX_SIZE : h;
 
+    // If request would be even, make it odd so player is centered.
+    if (w % 2 == 0) {
+        w += 1;
+    }
+
+    if (h % 2 == 0) {
+        h += 1;
+    }
+
     if (w != want_config[CONFIG_MAPWIDTH] || h != want_config[CONFIG_MAPHEIGHT]) {
         want_config[CONFIG_MAPWIDTH] = w;
         want_config[CONFIG_MAPHEIGHT] = h;
-        client_mapsize(w, h);
+        if (csocket.fd) {
+            client_mapsize(w, h);
+        }
     }
 }
 
@@ -100,7 +110,6 @@ void map_init(GtkWidget *window_root) {
     if (face_info.facesets[face_info.faceset].size != NULL) {
         image_size = atoi(face_info.facesets[face_info.faceset].size);
         map_image_size  = image_size; // These should be the same.
-        map_image_half_size = map_image_size / 2;
     } else {
         LOG(LOG_ERROR, "map_init", "Invalid faceset size from server");
     }
@@ -109,30 +118,16 @@ void map_init(GtkWidget *window_root) {
         int nx = map_image_size, ny = map_image_size;
         guint8 *png_tmp = rescale_rgba_data(pixmaps[0]->map_image, &nx, &ny, use_config[CONFIG_MAPSCALE]);
         // Try to affect pixmap[0] in-place, since it is referenced extensively.
-        pixmaps[0]->map_width = nx;
-        pixmaps[0]->map_height = ny;
         pixmaps[0]->icon_width = nx;
         pixmaps[0]->icon_height = ny;
+        pixmaps[0]->full_icon_width = nx;
+        pixmaps[0]->full_icon_height = ny;
         do_new_image(png_tmp, pixmaps[0]);
     }
 
     // Set map size based on window size and show widget.
     map_check_resize();
     gtk_widget_show(map_drawing_area);
-
-    switch (use_config[CONFIG_DISPLAYMODE]) {
-#ifdef HAVE_SDL
-    case CFG_DM_SDL:
-        init_SDL(map_drawing_area,0);
-        break;
-#endif
-
-#ifdef HAVE_OPENGL
-    case CFG_DM_OPENGL:
-        init_opengl(map_drawing_area);
-        break;
-#endif
-    }
 }
 
 /**
@@ -140,10 +135,10 @@ void map_init(GtkWidget *window_root) {
  * @param ax Map cell on-screen x-coordinate
  * @param ay Map cell on-screen y-coordinate
  */
-static void draw_pixmap(cairo_t *cr, PixmapInfo *pixmap, int ax, int ay) {
+static void draw_pixmap(cairo_t *cr, PixmapInfo *pixmap, int ax, int ay, int off_x, int off_y) {
     const int dest_x = ax * map_image_size;
     const int dest_y = ay * map_image_size;
-    cairo_set_source_surface(cr, pixmap->map_image, dest_x, dest_y);
+    cairo_set_source_surface(cr, pixmap->map_image, dest_x + global_offset_x + off_x, dest_y + global_offset_y + off_y);
     cairo_paint(cr);
 }
 
@@ -151,19 +146,14 @@ static void draw_smooth_pixmap(cairo_t* cr, PixmapInfo* pixmap,
         const int sx, const int sy, const int dx, const int dy) {
     const int src_x = map_image_size * sx;
     const int src_y = map_image_size * sy;
-    const int dest_x = map_image_size * dx;
-    const int dest_y = map_image_size * dy;
+    const int dest_x = map_image_size * dx + global_offset_x;
+    const int dest_y = map_image_size * dy + global_offset_y;
     cairo_set_source_surface(cr, pixmap->map_image, dest_x - src_x, dest_y - src_y);
     cairo_rectangle(cr, dest_x, dest_y, map_image_size, map_image_size);
     cairo_fill(cr);
 }
 
 int display_mapscroll(int dx, int dy) {
-#ifdef HAVE_SDL
-    if (use_config[CONFIG_DISPLAYMODE] == CFG_DM_SDL) {
-        return sdl_mapscroll(dx,dy);
-    } else
-#endif
         return 0;
 }
 
@@ -290,25 +280,26 @@ static void drawsmooth(cairo_t *cr, int mx, int my, int layer, int picx, int pic
 /**
  * Draw a single map layer to the given cairo context.
  */
-static void map_draw_layer(cairo_t *cr, int layer, int pass) {
-    for (int x = 0; x < use_config[CONFIG_MAPWIDTH]; x++) {
-        for (int y = 0; y < use_config[CONFIG_MAPHEIGHT]; y++) {
+static void map_draw_layer(cairo_t *cr, int layer, int pass, int mx_start, int nx, int my_start, int ny, int off_x, int off_y) {
+    for (int x = 0; x <= nx; x++) {
+        for (int y = 0; y <= ny; y++) {
             // Translate on-screen coordinates to virtual map coordinates.
-            int mx = pl_pos.x + x, my = pl_pos.y + y;
+            const int mx = mx_start + x;
+            const int my = my_start + y;
+
+            // Skip current cell if not visible and not using fog of war.
+            if (!use_config[CONFIG_FOGWAR] && mapdata_cell(mx, my)->cleared) {
+                continue;
+            }
 
             if (pass == 0) {
-                // Skip current cell if not visible and not using fog of war.
-                if (!use_config[CONFIG_FOGWAR] && mapdata_cell(mx, my)->cleared) {
-                    continue;
-                }
-
                 int dx, dy, face = mapdata_face_info(mx, my, layer, &dx, &dy);
                 if (face > 0 && pixmaps[face]->map_image != NULL) {
-                    draw_pixmap(cr, pixmaps[face], x + dx, y + dy);
+                    draw_pixmap(cr, pixmaps[face], x + dx, y + dy, off_x, off_y);
                 }
             } else if (pass == 1) {
                 if (use_config[CONFIG_SMOOTH]) {
-                    drawsmooth(cr, mx, my, layer, x, y);
+                    drawsmooth(cr, mx, my, layer, x + off_x, y + off_y);
                 }
             }
         }
@@ -319,7 +310,7 @@ static void map_draw_layer(cairo_t *cr, int layer, int pass) {
  * Draw darkness layer to a location on screen.
  */
 static void mapcell_draw_darkness(cairo_t *cr, int ax, int ay, int mx, int my) {
-    cairo_rectangle(cr, ax * map_image_size, ay * map_image_size,
+    cairo_rectangle(cr, ax * map_image_size + global_offset_x, ay * map_image_size + global_offset_y,
             map_image_size, map_image_size);
 
     double opacity = mapdata_cell(mx, my)->darkness / 192.0 * 0.6;
@@ -335,38 +326,74 @@ static void mapcell_draw_darkness(cairo_t *cr, int ax, int ay, int mx, int my) {
 /**
  * Redraw the entire map using GTK.
  */
-static void gtk_map_redraw(gboolean redraw) {
-    if (!redraw && !map_updated) {
+static void gtk_map_redraw() {
+    if (!map_updated) {
         return;
     }
 
     GtkAllocation size;
     gtk_widget_get_allocation(map_drawing_area, &size);
-    const int width = use_config[CONFIG_MAPWIDTH] * map_image_size;
-    const int height = use_config[CONFIG_MAPHEIGHT] * map_image_size;
+
+    // Effective dimensions in pixels, i.e. after adjusting for map scale
+    float scale = use_config[CONFIG_MAPSCALE]/100.0;
+    const double ew = size.width / scale;
+    const double eh = size.height / scale;
+
+    // Number of tiles to show in x and y dimensions
+    const int nx = (int)ceilf(ew / map_image_size);
+    const int ny = (int)ceilf(eh / map_image_size);
+
+    // Current viewport dimensions as sent by server, in squares
+    const int vw = use_config[CONFIG_MAPWIDTH];
+    const int vh = use_config[CONFIG_MAPHEIGHT];
+
+    // The server always centers the player in the viewport. However, if our
+    // drawing area shows more tiles than the viewport, then the player is
+    // no longer centered. Correct that here.
+    const int mx_start = (nx > vw) ? pl_pos.x - (nx - vw)/2 : pl_pos.x;
+    const int my_start = (ny > vh) ? pl_pos.y - (ny - vh)/2 : pl_pos.y;
 
     // Create double buffer and associated graphics context.
-    cairo_surface_t *cst =
-            cairo_image_surface_create(CAIRO_FORMAT_ARGB32, width, height);
+    cairo_surface_t *cst = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, ew, eh);
     cairo_t *cr = cairo_create(cst);
 
     // Blank graphics context with a solid black background.
     cairo_set_source_rgb(cr, 0, 0, 0);
-    cairo_rectangle(cr, 0, 0, width, height);
+    cairo_rectangle(cr, 0, 0, ew, eh);
     cairo_fill(cr);
 
     for (int layer = 0; layer < MAXLAYERS; layer++) {
-        map_draw_layer(cr, layer, 0);
-        map_draw_layer(cr, layer, 1);
+        map_draw_layer(cr, layer, 0, mx_start, nx, my_start, ny, 0, 0);
+        map_draw_layer(cr, layer, 1, mx_start, nx, my_start, ny, 0, 0);
     }
 
-    for (int x = 0; x < use_config[CONFIG_MAPWIDTH]; x++) {
-        for (int y = 0; y < use_config[CONFIG_MAPHEIGHT]; y++) {
-            // Determine the 'virtual' map coordinates.
-            int mx = pl_pos.x + x, my = pl_pos.y + y;
+    for (int x = 0; x <= nx; x++) {
+        for (int y = 0; y <= ny; y++) {
+            // Translate on-screen coordinates to virtual map coordinates.
+            const int mx = mx_start + x;
+            const int my = my_start + y;
+
             mapcell_draw_darkness(cr, x, y, mx, my);
             mapdata_cell(mx, my)->need_update = 0;
             mapdata_cell(mx, my)->need_resmooth = 0;
+
+            // Draw move-to tile.
+            if (mx == move_to_x && my == move_to_y && !is_at_moveto()) {
+                int sx = x * map_image_size;
+                int sy = y * map_image_size;
+                cairo_rectangle(cr, sx + global_offset_x, sy + global_offset_y, map_image_size, map_image_size);
+                if (move_to_attack) {
+                    cairo_set_source_rgb(cr, 1, 0, 0); // red to attack
+                } else {
+                    cairo_set_source_rgb(cr, 1, 1, 0); // yellow to walk
+                }
+                cairo_set_line_width(cr, 2);
+                cairo_stroke(cr);
+                cairo_rectangle(cr, sx + global_offset_x + 2, sy + global_offset_y + 2, map_image_size - 4, map_image_size - 4);
+                cairo_set_source_rgb(cr, 0, 0, 0); // black
+                cairo_set_line_width(cr, 1);
+                cairo_stroke(cr);
+            }
         }
     }
 
@@ -375,10 +402,12 @@ static void gtk_map_redraw(gboolean redraw) {
     // Copy the double buffer on the map drawing area.
     cairo_t *map_cr = gdk_cairo_create(gtk_widget_get_window(map_drawing_area));
     if (use_config[CONFIG_MAPSCALE] != 100) {
-        float scale = use_config[CONFIG_MAPSCALE]/100.0;
         cairo_scale(map_cr, scale, scale);
     }
     cairo_set_source_surface(map_cr, cst, 0, 0);
+    if (use_config[CONFIG_MAPSCALE] > 100) {
+        cairo_pattern_set_filter(cairo_get_source(cr), CAIRO_FILTER_NEAREST);
+    }
     cairo_paint(map_cr);
     cairo_destroy(map_cr);
 
@@ -394,45 +423,48 @@ static void gtk_map_redraw(gboolean redraw) {
 void resize_map_window(int x, int y) {
 }
 
+static void update_global_offset() {
+    int dx, dy;
+    dx = ((want_offset_x*map_image_size) - global_offset_x)*predict_alpha/100.0;
+    dy = ((want_offset_y*map_image_size) - global_offset_y)*predict_alpha/100.0;
+    global_offset_x += dx;
+    global_offset_y += dy;
+
+    // Snap global offset to zero when we're at the right square.
+    if (want_offset_x == 0 && want_offset_y == 0) {
+        global_offset_x = 0;
+        global_offset_y = 0;
+    }
+}
+
 /**
  * Draw the map window using the appropriate backend.
- * @param redraw If true, the entire screen must be redrawn.
  */
-void draw_map(gboolean redraw) {
+void draw_map() {
     gint64 t_start, t_end;
+    t_start = g_get_monotonic_time();
 
+    update_global_offset();
+    gtk_map_redraw();
+
+    t_end = g_get_monotonic_time();
+    gint64 elapsed = t_end - t_start;
     if (time_map_redraw) {
-        t_start = g_get_monotonic_time();
-    }
-
-    switch (use_config[CONFIG_DISPLAYMODE]) {
-#ifdef HAVE_SDL
-    case CFG_DM_SDL:
-        sdl_gen_map(redraw);
-        break;
-#endif
-
-#ifdef HAVE_OPENGL
-    case CFG_DM_OPENGL:
-        opengl_gen_map(redraw);
-        break;
-#endif
-
-    default:
-        gtk_map_redraw(redraw);
-        break;
-    }
-
-    if (time_map_redraw) {
-        t_end = g_get_monotonic_time();
-        gint64 elapsed = t_end - t_start;
         printf("profile/redraw,%"G_GINT64_FORMAT"\n", elapsed);
     }
+
+    const unsigned int target_redraw = 100000;
+    if (elapsed > target_redraw) {
+        LOG(LOG_DEBUG, "draw_map", "Increasing mapscale to %d to reduce draw time below %u us",
+                use_config[CONFIG_MAPSCALE], target_redraw);
+        use_config[CONFIG_MAPSCALE] += 5;
+        map_check_resize();
+    }
 }
 
 static gboolean map_expose_event(GtkWidget *widget, GdkEventExpose *event,
         gpointer user_data) {
-    draw_map(TRUE);
+    draw_map();
     return FALSE;
 }
 
@@ -442,7 +474,7 @@ static gboolean map_expose_event(GtkWidget *widget, GdkEventExpose *event,
  * @param dy Relative 'y' coordinate
  * @return 0 if x and y are both zero, 1-8 for each compass direction
  */
-static int relative_direction(int dx, int dy) {
+int relative_direction(int dx, int dy) {
     if (dx == 0 && dy == 0) {
         return 0;
     } else if (dx == 0 && dy < 0) {
@@ -466,14 +498,60 @@ static int relative_direction(int dx, int dy) {
     }
 }
 
+/**
+ * Given x, y coordinates on the map drawing area, determine the tile that the
+ * coordinates are on with respect to the player. Clicking on the tile with
+ * the player returns (0, 0).
+ */
+static void coord_to_tile(int x, int y, int *dx, int *dy) {
+    // Calculate tile below (x, y) coordinates. The top left corner is (0, 0).
+    // This is easy, just floor divide the coordinates by tile size.
+    const float tile_size = map_image_size * use_config[CONFIG_MAPSCALE]/100.0;
+    int mx = x / tile_size;
+    int my = y / tile_size;
+
+    // Now calculate where the player is drawn. The code below is copied from
+    // gtk_map_redraw() (see the comments there):
+    GtkAllocation size;
+    gtk_widget_get_allocation(map_drawing_area, &size);
+
+    float scale = use_config[CONFIG_MAPSCALE]/100.0;
+    const double ew = size.width / scale;
+    const double eh = size.height / scale;
+
+    const int nx = (int)ceilf(ew / map_image_size);
+    const int ny = (int)ceilf(eh / map_image_size);
+
+    const int vw = use_config[CONFIG_MAPWIDTH];
+    const int vh = use_config[CONFIG_MAPHEIGHT];
+
+    // Normally, the player is centered in the middle of the server viewport
+    // (with floor division).
+    int ox = vw / 2;
+    int oy = vh / 2;
+
+    // If mapscale is less than 100, what is shown in the client no longer
+    // matches the viewport. Add the right offset.
+    if (nx > vw) {
+        ox += (nx - vw)/2;
+    }
+
+    if (ny > vh) {
+        oy += (ny - vh)/2;
+    }
+
+    // Shift the clicked tile by the player's position.
+    *dx = mx - ox;
+    *dy = my - oy;
+}
+
 /**
  * Handle a mouse event in the drawing area.
  */
 static gboolean map_button_event(GtkWidget *widget,
         GdkEventButton *event, gpointer user_data) {
-    // Determine the tile of the mouse event, relative to the player.
-    int dx = ((int)event->x - 2) / map_image_size - (use_config[CONFIG_MAPWIDTH] / 2);
-    int dy = ((int)event->y - 2) / map_image_size - (use_config[CONFIG_MAPHEIGHT] / 2);
+    int dx, dy;
+    coord_to_tile((int)event->x, (int)event->y, &dx, &dy);
     int dir = relative_direction(dx, dy);
 
     switch (event->button) {
@@ -490,10 +568,9 @@ static gboolean map_button_event(GtkWidget *widget,
             }
             break;
         case 3:
-            if (event->type == GDK_BUTTON_RELEASE) {
-                stop_run();
-            } else {
-                run_dir(dir);
+            if (event->type == GDK_BUTTON_PRESS) {
+                set_move_to(dx, dy);
+                run_move_to();
             }
             break;
     }
